@@ -113,6 +113,43 @@ geocode_location_get_property (GObject    *object,
         }
 }
 
+/**
+ * geocode_location_equal:
+ * @a: a location
+ * @b: another location
+ *
+ * Compare two #GeocodeLocation instances for equality. This compares all fields
+ * and only returns %TRUE if the instances are exactly equal. For example, if
+ * both locations have the same physical coordinates, but one location has its
+ * #GeocodeLocation:description property set and the other does not, %FALSE
+ * will be returned. Similarly, if both locations have the same
+ * #GeocodeLocation:latitude, #GeocodeLocation:longitude and
+ * #GeocodeLocation:altitude, but a different #GeocodeLocation:accuracy or
+ * #GeocodeLocation:timestamp, %FALSE will be returned. Or if both locations
+ * have the same#GeocodeLocation:latitude and #GeocodeLocation:longitude but a
+ * different #GeocodeLocation:altitude, %FALSE will be returned.
+ *
+ * Both instances must be non-%NULL.
+ *
+ * Returns: %TRUE if the instances are equal, %FALSE otherwise
+ * Since: 3.23.1
+ */
+gboolean
+geocode_location_equal (GeocodeLocation *a,
+                        GeocodeLocation *b)
+{
+        g_return_val_if_fail (GEOCODE_IS_LOCATION (a), FALSE);
+        g_return_val_if_fail (GEOCODE_IS_LOCATION (b), FALSE);
+
+        return (a->priv->longitude == b->priv->longitude &&
+                a->priv->latitude == b->priv->latitude &&
+                a->priv->altitude == b->priv->altitude &&
+                a->priv->accuracy == b->priv->accuracy &&
+                a->priv->timestamp == b->priv->timestamp &&
+                g_strcmp0 (a->priv->description, b->priv->description) == 0 &&
+                a->priv->crs == b->priv->crs);
+}
+
 static void
 geocode_location_set_latitude (GeocodeLocation *loc,
                                gdouble          latitude)
@@ -156,6 +193,28 @@ geocode_location_set_crs(GeocodeLocation   *loc,
         loc->priv->crs = crs;
 }
 
+static void
+geocode_location_set_timestamp (GeocodeLocation *loc,
+                                guint64          timestamp)
+{
+        g_return_if_fail (GEOCODE_IS_LOCATION (loc));
+
+        loc->priv->timestamp = timestamp;
+}
+
+static void
+geocode_location_constructed (GObject *object)
+{
+        GeocodeLocation *location = GEOCODE_LOCATION (object);
+        GTimeVal tv;
+
+        if (location->priv->timestamp != 0)
+                return;
+
+        g_get_current_time (&tv);
+        geocode_location_set_timestamp (location, tv.tv_sec);
+}
+
 static void
 geocode_location_set_property(GObject      *object,
                               guint         property_id,
@@ -195,6 +254,11 @@ geocode_location_set_property(GObject      *object,
                                           g_value_get_enum (value));
                 break;
 
+        case PROP_TIMESTAMP:
+                geocode_location_set_timestamp (location,
+                                                g_value_get_uint64 (value));
+                break;
+
         default:
                 /* We don't have any other property... */
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -481,6 +545,7 @@ geocode_location_class_init (GeocodeLocationClass *klass)
         glocation_class->finalize = geocode_location_finalize;
         glocation_class->get_property = geocode_location_get_property;
         glocation_class->set_property = geocode_location_set_property;
+        glocation_class->constructed = geocode_location_constructed;
 
         g_type_class_add_private (klass, sizeof (GeocodeLocationPrivate));
 
@@ -578,7 +643,10 @@ geocode_location_class_init (GeocodeLocationClass *klass)
          * GeocodeLocation:timestamp:
          *
          * A timestamp in seconds since
-         * <ulink url="http://en.wikipedia.org/wiki/Unix_epoch">Epoch</ulink>.
+         * <ulink url="http://en.wikipedia.org/wiki/Unix_epoch">Epoch</ulink>,
+         * giving when the location was resolved from an address.
+         *
+         * A value of 0 (zero) will be interpreted as the current time.
          */
         pspec = g_param_spec_uint64 ("timestamp",
                                      "Timestamp",
@@ -587,7 +655,8 @@ geocode_location_class_init (GeocodeLocationClass *klass)
                                      0,
                                      G_MAXINT64,
                                      0,
-                                     G_PARAM_READABLE |
+                                     G_PARAM_READWRITE |
+                                     G_PARAM_CONSTRUCT_ONLY |
                                      G_PARAM_STATIC_STRINGS);
         g_object_class_install_property (glocation_class, PROP_TIMESTAMP, pspec);
 }
@@ -595,14 +664,10 @@ geocode_location_class_init (GeocodeLocationClass *klass)
 static void
 geocode_location_init (GeocodeLocation *location)
 {
-        GTimeVal tv;
-
         location->priv = G_TYPE_INSTANCE_GET_PRIVATE ((location),
                                                       GEOCODE_TYPE_LOCATION,
                                                       GeocodeLocationPrivate);
 
-        g_get_current_time (&tv);
-        location->priv->timestamp = tv.tv_sec;
         location->priv->altitude = GEOCODE_LOCATION_ALTITUDE_UNKNOWN;
         location->priv->accuracy = GEOCODE_LOCATION_ACCURACY_UNKNOWN;
         location->priv->crs = GEOCODE_LOCATION_CRS_WGS84;
@@ -804,14 +869,15 @@ geocode_location_get_crs (GeocodeLocation *loc)
  * geocode_location_get_timestamp:
  * @loc: a #GeocodeLocation
  *
- * Gets the timestamp (in seconds since Epoc) of location @loc.
+ * Gets the timestamp (in seconds since the Epoch) of location @loc. See
+ * #GeocodeLocation:timestamp.
  *
  * Returns: The timestamp of location @loc.
  **/
 guint64
 geocode_location_get_timestamp (GeocodeLocation *loc)
 {
-        g_return_val_if_fail (GEOCODE_IS_LOCATION (loc), -1);
+        g_return_val_if_fail (GEOCODE_IS_LOCATION (loc), 0);
 
         return loc->priv->timestamp;
 }
@@ -831,7 +897,7 @@ geo_uri_from_location (GeocodeLocation *loc)
         char *uri;
         char *coords;
         char *params;
-        char *crs = "wgs84";
+        const char *crs = "wgs84";
         char lat[G_ASCII_DTOSTR_BUF_SIZE];
         char lon[G_ASCII_DTOSTR_BUF_SIZE];
         char alt[G_ASCII_DTOSTR_BUF_SIZE];
