@@ -57,6 +57,7 @@ struct _GeocodePlacePrivate {
         char *country;
         char *continent;
         char *osm_id;
+        GeocodePlaceOsmType osm_type;
 };
 
 enum {
@@ -79,7 +80,8 @@ enum {
         PROP_CONTINENT,
         PROP_ICON,
         PROP_BBOX,
-        PROP_OSM_ID
+        PROP_OSM_ID,
+        PROP_OSM_TYPE
 };
 
 G_DEFINE_TYPE (GeocodePlace, geocode_place, G_TYPE_OBJECT)
@@ -183,6 +185,11 @@ geocode_place_get_property (GObject    *object,
                                     geocode_place_get_osm_id (place));
                 break;
 
+        case PROP_OSM_TYPE:
+                g_value_set_enum (value,
+                                  geocode_place_get_osm_type (place));
+                break;
+
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
                 break;
@@ -271,6 +278,10 @@ geocode_place_set_property(GObject      *object,
                 place->priv->osm_id = g_value_dup_string (value);
                 break;
 
+        case PROP_OSM_TYPE:
+                place->priv->osm_type = g_value_get_enum (value);
+                break;
+
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
                 break;
@@ -551,6 +562,19 @@ geocode_place_class_init (GeocodePlaceClass *klass)
                                      G_PARAM_STATIC_STRINGS);
         g_object_class_install_property (gplace_class, PROP_OSM_ID, pspec);
 
+        /**
+         * GeocodePlace:osm-type:
+         *
+         * The OpenStreetMap type of the place.
+         */
+        pspec = g_param_spec_enum ("osm-type",
+                                   "OSM Type",
+                                   "The OpenStreetMap type of the place",
+                                   GEOCODE_TYPE_PLACE_OSM_TYPE,
+                                   GEOCODE_PLACE_OSM_TYPE_UNKNOWN,
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_STATIC_STRINGS);
+        g_object_class_install_property (gplace_class, PROP_OSM_TYPE, pspec);
 }
 
 static void
@@ -560,6 +584,7 @@ geocode_place_init (GeocodePlace *place)
                                                       GEOCODE_TYPE_PLACE,
                                                       GeocodePlacePrivate);
         place->priv->bbox = NULL;
+        place->priv->osm_type = GEOCODE_PLACE_OSM_TYPE_UNKNOWN;
 }
 
 /**
@@ -603,6 +628,67 @@ geocode_place_new_with_location (const char      *name,
                              NULL);
 }
 
+/* NULL-safe #GeocodeLocation equality check. */
+static gboolean
+location_equal0 (GeocodeLocation *a,
+                 GeocodeLocation *b)
+{
+        return ((a == NULL && b == NULL) ||
+                (a != NULL && b != NULL && geocode_location_equal (a, b)));
+}
+
+/* NULL-safe #GeocodeBoundingBox equality check. */
+static gboolean
+bbox_equal0 (GeocodeBoundingBox *a,
+             GeocodeBoundingBox *b)
+{
+        return ((a == NULL && b == NULL) ||
+                (a != NULL && b != NULL && geocode_bounding_box_equal (a, b)));
+}
+
+/**
+ * geocode_place_equal:
+ * @a: a place
+ * @b: another place
+ *
+ * Compare two #GeocodePlace instances for equality. This compares all fields
+ * and only returns %TRUE if the instances are exactly equal. For example, if
+ * both places have the same #GeocodePlace:location, but place @b has its
+ * #GeocodePlace:continent property set and place @a does not, %FALSE will be
+ * returned.
+ *
+ * Both instances must be non-%NULL.
+ *
+ * Returns: %TRUE if the instances are equal, %FALSE otherwise
+ * Since: 3.23.1
+ */
+gboolean
+geocode_place_equal (GeocodePlace *a,
+                     GeocodePlace *b)
+{
+        g_return_val_if_fail (GEOCODE_IS_PLACE (a), FALSE);
+        g_return_val_if_fail (GEOCODE_IS_PLACE (b), FALSE);
+
+        return (g_strcmp0 (a->priv->name, b->priv->name) == 0 &&
+                a->priv->place_type == b->priv->place_type &&
+                location_equal0 (a->priv->location, b->priv->location) &&
+                bbox_equal0 (a->priv->bbox, b->priv->bbox) &&
+                g_strcmp0 (a->priv->street_address, b->priv->street_address) == 0 &&
+                g_strcmp0 (a->priv->street, b->priv->street) == 0 &&
+                g_strcmp0 (a->priv->building, b->priv->building) == 0 &&
+                g_strcmp0 (a->priv->postal_code, b->priv->postal_code) == 0 &&
+                g_strcmp0 (a->priv->area, b->priv->area) == 0 &&
+                g_strcmp0 (a->priv->town, b->priv->town) == 0 &&
+                g_strcmp0 (a->priv->county, b->priv->county) == 0 &&
+                g_strcmp0 (a->priv->state, b->priv->state) == 0 &&
+                g_strcmp0 (a->priv->admin_area, b->priv->admin_area) == 0 &&
+                g_strcmp0 (a->priv->country_code, b->priv->country_code) == 0 &&
+                g_strcmp0 (a->priv->country, b->priv->country) == 0 &&
+                g_strcmp0 (a->priv->continent, b->priv->continent) == 0 &&
+                g_strcmp0 (a->priv->osm_id, b->priv->osm_id) == 0 &&
+                a->priv->osm_type == b->priv->osm_type);
+}
+
 /**
  * geocode_place_set_name:
  * @place: A place
@@ -1097,59 +1183,46 @@ geocode_place_get_continent (GeocodePlace *place)
         return place->priv->continent;
 }
 
-static char *
+static const char *
 get_icon_name (GeocodePlace *place)
 {
-        char *icon_name;
-
-        switch (place->priv->place_type) {
-
+        switch ((int) place->priv->place_type) {
         case GEOCODE_PLACE_TYPE_BUILDING:
-                icon_name = "poi-building";
-                break;
+                return "poi-building";
 
         case GEOCODE_PLACE_TYPE_TOWN:
-                icon_name = "poi-town";
-                break;
+                return "poi-town";
 
         case GEOCODE_PLACE_TYPE_AIRPORT:
-                icon_name = "poi-airport";
-                break;
+                return "poi-airport";
 
         case GEOCODE_PLACE_TYPE_RAILWAY_STATION:
-                icon_name = "poi-railway-station";
-                break;
+                return "poi-railway-station";
 
         case GEOCODE_PLACE_TYPE_BUS_STOP:
-                icon_name = "poi-bus-stop";
-                break;
+                return "poi-bus-stop";
 
         case GEOCODE_PLACE_TYPE_STREET:
-                icon_name = "poi-car";
-                break;
+                return "poi-car";
 
         case GEOCODE_PLACE_TYPE_SCHOOL:
-                icon_name = "poi-school";
-                break;
+                return "poi-school";
 
         case GEOCODE_PLACE_TYPE_PLACE_OF_WORSHIP:
-                icon_name = "poi-place-of-worship";
-                break;
+                return "poi-place-of-worship";
 
         case GEOCODE_PLACE_TYPE_RESTAURANT:
-                icon_name = "poi-restaurant";
-                break;
+                return "poi-restaurant";
 
         case GEOCODE_PLACE_TYPE_BAR:
-                icon_name = "poi-bar";
-                break;
+                return "poi-bar";
+
+        case GEOCODE_PLACE_TYPE_LIGHT_RAIL_STATION:
+                return "poi-light-rail-station";
 
         default:
-                icon_name = "poi-marker"; /* generic marker */
-                break;
+                return "poi-marker"; /* generic marker */
         }
-
-        return icon_name;
 }
 
 /**
@@ -1163,7 +1236,7 @@ get_icon_name (GeocodePlace *place)
 GIcon *
 geocode_place_get_icon (GeocodePlace *place)
 {
-        char *icon_name;
+        const char *icon_name;
 
         g_return_val_if_fail (GEOCODE_IS_PLACE (place), NULL);
 
@@ -1223,3 +1296,19 @@ geocode_place_get_osm_id (GeocodePlace *place)
 
         return place->priv->osm_id;
 }
+
+/**
+ * geocode_place_get_osm_type:
+ * @place: A place
+ *
+ * Gets the OpenStreetMap type of the @place.
+ *
+ * Returns: The osm type of the @place.
+ **/
+GeocodePlaceOsmType
+geocode_place_get_osm_type (GeocodePlace *place)
+{
+        g_return_val_if_fail (GEOCODE_IS_PLACE (place), GEOCODE_PLACE_OSM_TYPE_UNKNOWN);
+
+        return place->priv->osm_type;
+}
